from django.contrib import admin

from ..admin_site import intecomm_screening_admin
from ..models import Site


@admin.register(Site, site=intecomm_screening_admin)
class SiteAdmin(admin.ModelAdmin):
    search_fields = (
        "id",
        "name",
    )



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import intecomm_subject_admin
from ..forms import HealthEconomicsForm
from ..models import HealthEconomics
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(HealthEconomics, site=intecomm_subject_admin)
class HealthEconomicsAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):

    form = HealthEconomicsForm
    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Part 1: Education",
            {
                "fields": (
                    "occupation",
                    "education_in_years",
                    "education_certificate",
                    "primary_school",
                    "primary_school_in_years",
                    "secondary_school",
                    "secondary_school_in_years",
                    "higher_education",
                    "higher_education_in_years",
                )
            },
        ),
        (
            "Part 2: Income",
            {"fields": ("welfare",)},
        ),
        (
            "Part 3: Previous Healthcare Expenses: Medication",
            {"fields": ("received_rx_month",)},
        ),
        (
            "Part 3a: Previous Healthcare Expenses: Medication (Diabetes - DM)",
            {
                "fields": (
                    "rx_dm_month",
                    "rx_dm_paid_month",
                    "rx_dm_paid_month_other",
                    "rx_dm_cost_month",
                )
            },
        ),
        (
            "Part 3b: Previous Healthcare Expenses: Medication (Hypertension - HTN)",
            {
                "fields": (
                    "rx_htn_month",
                    "rx_htn_paid_month",
                    "rx_htn_paid_month_other",
                    "rx_htn_cost_month",
                )
            },
        ),
        (
            "Part 3c: Previous Healthcare Expenses: Medication (HIV)",
            {
                "fields": (
                    "rx_hiv_month",
                    "rx_hiv_paid_month",
                    "rx_hiv_paid_month_other",
                    "rx_hiv_cost_month",
                )
            },
        ),
        (
            "Part 3d: Previous Healthcare Expenses: Other Medications",
            {
                "fields": (
                    "rx_other_month",
                    "rx_other_paid_month",
                    "rx_other_paid_month_other",
                    "rx_other_cost_month",
                )
            },
        ),
        (
            "Part 4: Current Visit Healthcare Expenses: Medications",
            {"fields": ("received_rx_today",)},
        ),
        (
            "Part 4a: Current Visit Healthcare Expenses: Medications (Diabetes - DM)",
            {
                "fields": (
                    "rx_dm_today",
                    "rx_dm_paid_today",
                    "rx_dm_paid_today_other",
                    "rx_dm_cost_today",
                )
            },
        ),
        (
            "Part 4b: Current Visit Healthcare Expenses: Medications (Hypertension - HTN)",
            {
                "fields": (
                    "rx_htn_today",
                    "rx_htn_paid_today",
                    "rx_htn_paid_today_other",
                    "rx_htn_cost_today",
                )
            },
        ),
        (
            "Part 4c: Current Visit Healthcare Expenses: Medications (HIV)",
            {
                "fields": (
                    "rx_hiv_today",
                    "rx_hiv_paid_today",
                    "rx_hiv_paid_today_other",
                    "rx_hiv_cost_today",
                )
            },
        ),
        (
            "Part 4d: Current Visit Healthcare Expenses: Other Medications",
            {
                "fields": (
                    "rx_other_today",
                    "rx_other_paid_today",
                    "rx_other_paid_today_other",
                    "rx_other_cost_today",
                )
            },
        ),
        (
            "Part 5: Health Care Financing",
            {
                "fields": (
                    "health_insurance",
                    "health_insurance_cost",
                    "patient_club",
                    "patient_club_cost",
                )
            },
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "crf_status": admin.VERTICAL,
        "health_insurance": admin.VERTICAL,
        "higher_education": admin.VERTICAL,
        "patient_club": admin.VERTICAL,
        "primary_school": admin.VERTICAL,
        "received_rx_month": admin.VERTICAL,
        "received_rx_today": admin.VERTICAL,
        "rx_dm_month": admin.VERTICAL,
        "rx_dm_today": admin.VERTICAL,
        "rx_hiv_month": admin.VERTICAL,
        "rx_hiv_today": admin.VERTICAL,
        "rx_htn_month": admin.VERTICAL,
        "rx_htn_today": admin.VERTICAL,
        "rx_other_month": admin.VERTICAL,
        "rx_other_today": admin.VERTICAL,
        "secondary_school": admin.VERTICAL,
        "welfare": admin.VERTICAL,
    }

    filter_horizontal = [
        "rx_dm_paid_month",
        "rx_htn_paid_month",
        "rx_hiv_paid_month",
        "rx_other_paid_month",
        "rx_dm_paid_today",
        "rx_htn_paid_today",
        "rx_hiv_paid_today",
        "rx_other_paid_today",
    ]



APPT = "appt"
APPT_OTHER = "other_routine_appt"
COMMUNITY_CLINIC = "community_clinic"
DIET_LIFESTYLE = "diet_lifestyle"
DRUGS = "drugs"
GTE_3HRS = "gte_3hr"
HEALTH_FACILITY = "health_facility"
HIV = "hiv"
INSULIN = "insulin"
INTEGRATED = "integrated"
MISSED_VISIT_ACTION = "missed_visit_action"
NCD = "ncd"
NURSE = "nurse"
SITTING = "sitting"
THIS_CLINIC = "this_clinic"
VERTICAL = "vertical"
VIT_B_FOLIC_ACID = "vitamin_b_folic_acid"



from django_audit_fields import ModelAdminAuditFieldsMixin
from django_revision.modeladmin_mixin import ModelAdminRevisionMixin
from edc_model_admin.history import SimpleHistoryAdmin
from edc_model_admin.mixins import (
    ModelAdminFormAutoNumberMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    TemplatesModelAdminMixin,
)
from edc_sites.modeladmin_mixins import SiteModelAdminMixin


class BaseModelAdminMixin(
    TemplatesModelAdminMixin,
    ModelAdminFormInstructionsMixin,
    ModelAdminFormAutoNumberMixin,
    ModelAdminRevisionMixin,
    ModelAdminInstitutionMixin,
    ModelAdminNextUrlRedirectMixin,
    ModelAdminAuditFieldsMixin,
    SiteModelAdminMixin,
    SimpleHistoryAdmin,
):
    show_cancel = True
    view_on_site = False
    save_on_top = True



from django import forms
from edc_constants.constants import HIV
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator
from edc_visit_schedule.utils import raise_if_baseline

from ..models import ClinicalReview


class ClinicalReviewFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_baseline(self.cleaned_data.get("subject_visit"))
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.validate_drawn_date_by_dx_date(HIV, "HIV infection")


class ClinicalReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ClinicalReviewFormValidator

    class Meta:
        model = ClinicalReview
        fields = "__all__"



from edc_visit_schedule import Crf, FormsCollection

# TODO: whats the difference between bloodresultsglu and glucose??

crfs_prn = FormsCollection(
    Crf(show_order=10, model="intecomm_subject.bloodresultsfbc"),
    Crf(show_order=150, model="intecomm_subject.glucose"),
    Crf(show_order=220, model="intecomm_subject.bloodresultsglu"),
    Crf(show_order=230, model="intecomm_subject.bloodresultshba1c"),
    Crf(show_order=240, model="intecomm_subject.bloodresultsrft"),
    Crf(show_order=250, model="intecomm_subject.bloodresultslft"),
    Crf(show_order=260, model="intecomm_subject.bloodresultslipid"),
    Crf(show_order=280, model="intecomm_subject.malariatest"),
    Crf(show_order=290, model="intecomm_subject.urinedipsticktest"),
    Crf(show_order=360, model="intecomm_subject.concomitantmedication"),
    Crf(show_order=380, model="intecomm_subject.urinepregnancy"),
    name="prn",
)

crfs_d1 = FormsCollection(
    Crf(show_order=100, model="intecomm_subject.clinicalreviewbaseline"),
    Crf(show_order=110, model="intecomm_subject.vitals"),
    Crf(show_order=120, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=140, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=143, model="intecomm_subject.medications"),
    Crf(show_order=145, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=150, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=155, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=160, model="intecomm_subject.otherbaselinedata"),
    Crf(show_order=165, model="intecomm_subject.complicationsbaseline"),
    name="day1",
)

crfs_1m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    Crf(show_order=220, model="intecomm_subject.familyhistory"),
    name="2m",
)
crfs_2m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="2m",
)
crfs_3m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="3m",
)
crfs_4m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="4m",
)
crfs_5m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="5m",
)
crfs_6m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="6m",
)
crfs_7m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="7m",
)
crfs_8m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="8m",
)
crfs_9m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="9m",
)
crfs_10m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="10m",
)
crfs_11m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=130, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=140, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=150, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=155, model="intecomm_subject.medications"),
    Crf(show_order=160, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=180, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="11m",
)
crfs_12m = FormsCollection(
    Crf(show_order=110, model="intecomm_subject.clinicalreview"),
    Crf(show_order=115, model="intecomm_subject.vitals"),
    Crf(show_order=112, model="intecomm_subject.hivinitialreview", required=False),
    Crf(show_order=114, model="intecomm_subject.dminitialreview", required=False),
    Crf(show_order=116, model="intecomm_subject.htninitialreview", required=False),
    Crf(show_order=120, model="intecomm_subject.hivreview", required=False),
    Crf(show_order=130, model="intecomm_subject.dmreview", required=False),
    Crf(show_order=140, model="intecomm_subject.htnreview", required=False),
    Crf(show_order=145, model="intecomm_subject.medications"),
    Crf(show_order=150, model="intecomm_subject.drugrefillhtn", required=False),
    Crf(show_order=160, model="intecomm_subject.drugrefilldm", required=False),
    Crf(show_order=170, model="intecomm_subject.drugrefillhiv", required=False),
    Crf(show_order=185, model="intecomm_subject.hivmedicationadherence", required=False),
    Crf(show_order=190, model="intecomm_subject.dmmedicationadherence", required=False),
    Crf(show_order=195, model="intecomm_subject.htnmedicationadherence", required=False),
    Crf(show_order=200, model="intecomm_subject.complicationsfollowup", required=False),
    name="12m",
)

crfs_unscheduled = FormsCollection(
    # Crf(show_order=10, model="intecomm_subject.followupvitals"),
    name="unscheduled",
)

crfs_missed = FormsCollection(
    Crf(show_order=10, model="intecomm_subject.subjectvisitmissed"),
    name="missed",
)



from django.contrib import admin
from edc_model_admin.mixins import TabularInlineMixin
from edc_rx.modeladmin_mixins import DrugRefillAdminMixin, DrugSupplyInlineMixin

from ..admin_site import intecomm_subject_admin
from ..forms import DrugRefillHivForm, DrugSupplyHivForm
from ..models import DrugRefillHiv, DrugSupplyHiv
from .modeladmin_mixins import CrfModelAdmin


class DrugSupplyHivInline(DrugSupplyInlineMixin, TabularInlineMixin, admin.TabularInline):
    model = DrugSupplyHiv
    form = DrugSupplyHivForm


@admin.register(DrugRefillHiv, site=intecomm_subject_admin)
class DrugRefillHivAdmin(DrugRefillAdminMixin, CrfModelAdmin):
    form = DrugRefillHivForm
    autocomplete_fields = ["rx"]

    filter_horizontal = ["modifications", "modifications_reason"]



import re
from typing import List
from uuid import uuid4

from django.db.models import Q
from edc_listboard.views import SubjectListboardView as BaseSubjectListboardView


class InteSubjectListboardView(BaseSubjectListboardView):

    listboard_template = "inte_subject_listboard_template"
    listboard_url = "inte_subject_listboard_url"
    navbar_selected_item = "inte_followup"
    search_form_url = "inte_subject_listboard_url"

    def get_queryset(self):
        # TODO
        qs = super().get_queryset()
        return qs.filter(id=uuid4())

    def extra_search_options(self, search_term) -> List[Q]:
        q_objects = super().extra_search_options(search_term)
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(identity__exact=search_term))
        return q_objects



from edc_constants.constants import NOT_APPLICABLE, OTHER

REFUSAL_REASONS = (
    ("dont_have_time", "I don't have time"),
    ("must_consult_spouse", "I need to consult my spouse"),
    ("dont_want_blood_drawn", "I don't want to have the blood drawn"),
    ("dont_want_to_join", "I don't want to take part"),
    ("need_to_think_about_it", "I haven't had a chance to think about it"),
    (OTHER, "Other, please specify"),
)

RESPONDENT_CHOICES = (
    ("patient", "Patient"),
    ("family", "Family"),
    ("friend", "friend"),
    ("other", "other"),
    (NOT_APPLICABLE, "Not applicable"),
)



from edc_visit_schedule import Schedule

from ..constants import COMM_SCHEDULE
from .visits import (
    visit00,
    visit01,
    visit02,
    visit03,
    visit04,
    visit05,
    visit06,
    visit07,
    visit08,
    visit09,
    visit10,
    visit11,
    visit12,
)

comm_schedule = Schedule(
    name=COMM_SCHEDULE,
    verbose_name="community-based integrated care",
    onschedule_model="intecomm_prn.onschedulecomm",
    offschedule_model="intecomm_prn.offschedulecomm",
    consent_model="intecomm_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
)


for visit in [
    visit00,
    visit01,
    visit02,
    visit03,
    visit04,
    visit05,
    visit06,
    visit07,
    visit08,
    visit09,
    visit10,
    visit11,
    visit12,
]:
    comm_schedule.add_visit(visit=visit)



from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin


class GroupingView(EdcViewMixin, NavbarViewMixin, TemplateView):

    template_name = f"intecomm_edc/bootstrap{settings.EDC_BOOTSTRAP}/grouping.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "screen_group"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        return context



screening_codenames = [
    "intecomm_screening.add_subjectrefusal",
    "intecomm_screening.change_subjectrefusal",
    "intecomm_screening.delete_subjectrefusal",
    "intecomm_screening.add_subjectscreening",
    "intecomm_screening.change_subjectscreening",
    "intecomm_screening.delete_subjectscreening",
    "intecomm_screening.view_historicalsubjectscreening",
    "intecomm_screening.view_historicalsubjectrefusal",
    "intecomm_screening.view_subjectscreening",
    "intecomm_screening.view_subjectrefusal",
]
screening_codenames.sort()



from django import forms
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.form_validator_mixins import ClinicalReviewBaselineFormValidatorMixin
from edc_form_validators import FormValidator

from ..models import ClinicalReviewBaseline


class ClinicalReviewBaselineFormValidator(
    ClinicalReviewBaselineFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    pass


class ClinicalReviewBaselineForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = ClinicalReviewBaselineFormValidator

    class Meta:
        model = ClinicalReviewBaseline
        fields = "__all__"



from __future__ import annotations

from uuid import uuid4

from django.db import models
from django.utils.html import format_html
from django_crypto_fields.fields import EncryptedCharField, EncryptedTextField
from edc_constants.choices import GENDER, YES_NO_TBD
from edc_constants.constants import DM, HIV, HTN, TBD
from edc_model.models import BaseUuidModel, HistoricalRecords, NameFieldsModelMixin
from edc_model.validators.phone import phone_number
from edc_model_fields.fields import InitialsField
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from intecomm_lists.models import Conditions

from .proxy_models import Site


class PatientLogManager(models.Manager):

    use_in_migrations = True

    def get_by_natural_key(self, legal_name):
        return self.get(legal_name=legal_name)


def abbrev_cond(c: list | None) -> str:
    c = sorted(c, key=str.casefold)
    if c == [DM]:
        abbrev = "*d*"
    elif c == [DM, HIV]:
        abbrev = "hd*"
    elif c == [DM, HTN]:
        abbrev = "*dt"
    elif c == [DM, HIV, HTN]:
        abbrev = "hdt"
    elif c == [HIV]:
        abbrev = "h**"
    elif c == [HIV, HTN]:
        abbrev = "h*t"
    elif c == [HTN]:
        abbrev = "**t"
    elif not c:
        abbrev = "***"
    else:
        raise TypeError(f"Invalid list of conditions. Got c == {c}.")
    return abbrev


class PatientLog(SiteModelMixin, NameFieldsModelMixin, BaseUuidModel):

    patient_log_identifier = models.CharField(
        max_length=36,
        default=uuid4,
        unique=True,
    )

    screening_identifier = models.CharField(
        max_length=25,
        null=True,
        blank=True,
        help_text="Auto populated when screening form is complete",
    )

    screening_datetime = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Auto populated when screening form is complete",
    )

    subject_identifier = models.CharField(
        max_length=25,
        null=True,
        blank=True,
        help_text="Auto populated when consent form is complete",
    )

    consent_datetime = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Auto populated when consent form is complete",
    )

    group_identifier = models.CharField(
        max_length=25,
        null=True,
        blank=True,
        help_text="Auto populated when group is randimized",
    )

    initials = InitialsField()

    gender = models.CharField(choices=GENDER, max_length=10, blank=False)

    age_in_years = models.IntegerField(verbose_name="Age")

    report_datetime = models.DateTimeField(default=get_utcnow)

    site = models.ForeignKey(
        Site,
        verbose_name="Health center",
        on_delete=models.PROTECT,
        blank=False,
        related_name="+",
    )

    hospital_identifier = EncryptedCharField(
        verbose_name="Hospital identifier",
        unique=True,
        blank=False,
        help_text="Must be unique",
    )

    last_4_hospital_identifier = EncryptedCharField(
        verbose_name="Last 4 digits of hospital_identifier",
        null=True,
        editable=False,
        help_text="auto-populated",
    )

    contact_number = EncryptedCharField(
        blank=False, validators=[phone_number], help_text="If unknown, type 'UNKNOWN'"
    )

    alt_contact_number = EncryptedCharField(null=True, blank=True, validators=[phone_number])

    last_4_contact_number = EncryptedCharField(
        verbose_name="Last 4 digits of contact number",
        null=True,
        editable=False,
        help_text="auto-populated",
    )

    may_contact = models.CharField(
        verbose_name="Has the patient agreed to be contacted prior to consent?",
        max_length=15,
        choices=YES_NO_TBD,
        default=TBD,
    )

    location_description = EncryptedTextField(
        null=True, blank=True, help_text="Street, landmarks near home, etc"
    )

    conditions = models.ManyToManyField(
        Conditions,
        verbose_name="Diagnoses",
    )

    stable = models.CharField(
        verbose_name="Do the facility health care staff consider the patient stable in care",
        max_length=15,
        choices=YES_NO_TBD,
        default=TBD,
        help_text="Refer to the SOP for the definition of 'stable'.",
    )

    last_appt_date = models.DateField(
        verbose_name="When was the patient last seen at this health facility",
        null=True,
        blank=True,
        help_text=(
            "If known, the last appointment may help to estimate the next appointment or "
            "the expected frequency of routine appointments"
        ),
    )

    next_appt_date = models.DateField(
        verbose_name="Next scheduled routine appointment at this health facility",
        help_text="This date will help prioritize efforts to contact the patient",
    )

    first_health_talk = models.CharField(
        verbose_name="Attended general health talk",
        max_length=15,
        choices=YES_NO_TBD,
        default=TBD,
    )

    first_health_talk_date = models.DateField(
        verbose_name="General talk date",
        null=True,
        blank=True,
    )

    second_health_talk = models.CharField(
        verbose_name="Attended sensitisation session",
        max_length=15,
        choices=YES_NO_TBD,
        default=TBD,
    )

    second_health_talk_date = models.DateField(
        verbose_name="Sensitisation session date", null=True, blank=True
    )

    call_attempts = models.IntegerField(default=0, help_text="auto-updated", blank=True)

    on_site = CurrentSiteManager()
    objects = PatientLogManager()
    history = HistoricalRecords()

    def __str__(self):
        return format_html(f"{self.legal_name.upper()}-{self.contact_number[-4:]}")

    def save(self, *args, **kwargs):
        self.legal_name = self.legal_name.upper()
        self.familiar_name = self.familiar_name.upper()
        self.initials = self.initials.upper()
        self.last_4_contact_number = self.contact_number[-4:]
        self.last_4_hospital_identifier = self.hospital_identifier[-4:]
        super().save(*args, **kwargs)

    def natural_key(self):
        return (self.legal_name,)  # noqa

    @property
    def patient_group(self):
        return self.patientgroup_set.all().first()


class Meta(SiteModelMixin.Meta, BaseUuidModel.Meta):
    verbose_name = "Patient Log"
    verbose_name_plural = "Patient Log"



from django.conf import settings
from django.views.generic import TemplateView
from edc_dashboard.view_mixins import EdcViewMixin
from edc_navbar import NavbarViewMixin


class FollowupInteView(EdcViewMixin, NavbarViewMixin, TemplateView):

    template_name = f"intecomm_edc/bootstrap{settings.EDC_BOOTSTRAP}/followup_inte.html"
    navbar_name = settings.APP_NAME
    navbar_selected_item = "inte_followup"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(
            inte_subject_listboard_url="intecomm_dashboard:inte_subject_listboard_url"
        )
        return context



# from django.contrib import admin
# from django.core.exceptions import ObjectDoesNotExist
# from django.template.loader import render_to_string
# from django.urls import reverse
# from django.utils.html import format_html
# from django_audit_fields import audit_fieldset_tuple
# from edc_constants.choices import GENDER
#
# from intecomm_screening.admin import (
#     AddPatientCallInline,
#     BaseModelAdminMixin,
#     ViewPatientCallInline,
# )
# from intecomm_screening.admin.list_filters import (
#     AttendDateListFilter,
#     DxListFilter,
#     LastApptListFilter,
#     NextApptListFilter,
# )
# from intecomm_screening.forms import PatientLogForm as BaseForm
#
# from ..admin_site import intecomm_group_admin
# from ..models import PatientGroup, PatientLog
#
#
# class PatientLogForm(BaseForm):
#     class Meta:
#         model = PatientLog
#
#
# @admin.register(PatientLog, site=intecomm_group_admin)
# class PatientLogAdmin(BaseModelAdminMixin):
#
#     form = PatientLogForm
#     list_per_page = 20
#     show_object_tools = True
#     show_cancel = True
#     change_list_template: str = "intecomm_group/admin/patientlog_change_list.html"
#     change_list_title = PatientLog._meta.verbose_name
#     change_list_note = format_html(
#         "In addition to other values, you may search for patients on the last 4-digits of "
#         "either their mobile number or hospital identifier."
#     )
#     change_list_help = (
#         "Searches on encrypted data work on exact uppercase matches only. When "
#         'searching on a full name, put the full name in quotations, for '
#         'example, "JOHN SMITH".'
#     )
#
#     autocomplete_fields = ["site"]
#
#     inlines = [AddPatientCallInline, ViewPatientCallInline]
#
#     fieldsets = (
#         (
#             None,
#             {
#                 "fields": (
#                     "report_datetime",
#                     "site",
#                 )
#             },
#         ),
#         (
#             "Name and basic demographics",
#             {
#                 "fields": (
#                     "legal_name",
#                     "familiar_name",
#                     "initials",
#                     "hospital_identifier",
#                     "gender",
#                     "age_in_years",
#                 )
#             },
#         ),
#         (
#             "Contact",
#             {
#                 "fields": (
#                     "contact_number",
#                     "alt_contact_number",
#                     "may_contact",
#                 )
#             },
#         ),
#         (
#             "Address / Location",
#             {"fields": ("location_description",)},
#         ),
#         (
#             "Appointments",
#             {
#                 "fields": (
#                     "last_appt_date",
#                     "next_appt_date",
#                 )
#             },
#         ),
#         (
#             "Screening and Consent",
#             {
#                 "classes": ("collapse",),
#                 "fields": (
#                     "screening_identifier",
#                     "screening_datetime",
#                     "subject_identifier",
#                     "consent_datetime",
#                 ),
#             },
#         ),
#         audit_fieldset_tuple,
#     )
#
#     list_display = (
#         "__str__",
#         "hf_id",
#         "group_name",
#         "appts",
#         "contacts",
#         "site_id",
#         "user_created",
#         "created",
#         "modified",
#         "screening_identifier",
#         "subject_identifier",
#     )
#
#     list_filter = (
#         "report_datetime",
#         "call_attempts",
#         DxListFilter,
#         AttendDateListFilter,
#         NextApptListFilter,
#         LastApptListFilter,
#         "first_health_talk",
#         "second_health_talk",
#         "gender",
#     )
#
#     filter_horizontal = ("conditions",)
#
#     search_fields = (
#         "id",
#         "screening_identifier",
#         "subject_identifier",
#         "last_4_hospital_identifier__exact",
#         "last_4_contact_number__exact",
#         "hospital_identifier__exact",
#         "initials__exact",
#         "legal_name__exact",
#         "familiar_name__exact",
#         "contact_number__exact",
#         "alt_contact_number__exact",
#     )
#
#     radio_fields = {
#         "first_health_talk": admin.VERTICAL,
#         "gender": admin.VERTICAL,
#         "may_contact": admin.VERTICAL,
#         "second_health_talk": admin.VERTICAL,
#         "stable": admin.VERTICAL,
#     }
#
#     readonly_fields = (
#         "screening_identifier",
#         "screening_datetime",
#         "subject_identifier",
#         "consent_datetime",
#         "legal_name",
#         "familiar_name",
#         "initials",
#         "hospital_identifier",
#         "gender",
#         "age_in_years",
#         "report_datetime",
#         "site",
#     )
#
#     def post_url_on_delete_kwargs(self, request, obj):
#         return {}
#
#     @admin.display(description="Date logged", ordering="report_datetime")
#     def date_logged(self, obj=None):
#         return obj.report_datetime.date()
#
#     @admin.display(description="next_appt", ordering="next_appt_date")
#     def next_appt(self, obj=None):
#         return obj.next_appt_date
#
#     @admin.display(description="last_appt", ordering="last_appt_date")
#     def last_appt(self, obj=None):
#         return obj.last_appt_date
#
#     @admin.display(description="HF ID", ordering="hospital_identifier")
#     def hf_id(self, obj=None):
#         context = dict(hospital_identifier=obj.hospital_identifier)
#         return format_html(
#             render_to_string(
#                 "intecomm_screening/change_list_hospital_identifier.html", context=context
#             )
#         )
#
#     @admin.display(description="Appts", ordering="next_appt_date")
#     def appts(self, obj=None):
#         context = dict(
#             last_appt=obj.last_appt_date or "-",
#             next_appt=obj.next_appt_date or "-",
#         )
#         return format_html(
#             render_to_string("intecomm_screening/change_list_appts.html", context=context)
#         )
#
#     @admin.display(description="Contacts", ordering="contact_number")
#     def contacts(self, obj=None):
#         add_patient_call_url = reverse(
#             "intecomm_screening_admin:intecomm_screening_patientcall_add"
#         )
#         patient_call_url = reverse(
#             "intecomm_screening_admin:intecomm_screening_patientcall_changelist"
#         )
#         patient_call_url = f"{patient_call_url}?q={obj.id}"
#         context = dict(
#             patient_log_id=obj.id,
#             add_patient_call_url=add_patient_call_url,
#             patient_call_url=patient_call_url,
#             contact_number=obj.contact_number,
#             alt_contact_number=obj.alt_contact_number,
#             call_attempts=obj.call_attempts,
#         )
#         return format_html(
#             render_to_string("intecomm_screening/change_list_contacts.html", context=context)
#         )
#
#     @admin.display(description="Site", ordering="site")
#     def site_id(self, obj=None):
#         return obj.site.id
#
#     @admin.display(description="Patient", ordering="familiar_name")
#     def patient(self, obj=None):
#         return f"{obj.familiar_name} ({obj.initials})"
#
#     @admin.display(description="Group", ordering="patientgroup__name")
#     def group_name(self, obj=None):
#         context = dict()
#         if obj.patientgroup_set.all().count() > 0:
#             patient_group = obj.patientgroup_set.all().first()
#             url = reverse("intecomm_group_admin:intecomm_group_patientgroup_changelist")
#             url = f"{url}?q={patient_group.name}"
#             context.update(url=url, patient_group=patient_group)
#         return format_html(
#             render_to_string("intecomm_group/change_list_group.html", context=context)
#         )
#
#     @admin.display(description="Calls", ordering="contact_attempts")
#     def calls(self, obj):
#         url = reverse("intecomm_screening_admin:intecomm_screening_patientcall_changelist")
#         url = f"{url}?q={obj.patientcall.id}"
#         return format_html(f'<A href="{url}">{obj.contact_attempts}</a>')
#
#     def get_search_results(self, request, queryset, search_term):
#         """Union initial search queryset (qs1) with
#         patients in a group whose name matches the search term (qs2).
#
#         Note: queryset is a queryset already passed through the
#         changelist's filters.
#         """
#         qs1, may_have_duplicates = super().get_search_results(
#             request,
#             queryset,
#             search_term,
#         )
#         queryset_pks = [obj.pk for obj in queryset.all()]
#         try:
#             patient_group = PatientGroup.objects.get(name__iexact=search_term)
#         except ObjectDoesNotExist:
#             qs = qs1
#         else:
#             pks = [obj.pk for obj in patient_group.patients.filter(pk__in=queryset_pks)]
#             qs = qs1 | self.model.objects.filter(id__in=pks)
#         return qs, True



import re
from typing import List

from django.db.models import Q
from edc_listboard.views import SubjectListboardView as BaseSubjectListboardView

from ....model_wrappers import SubjectConsentModelWrapper


class CommSubjectListboardView(BaseSubjectListboardView):

    listboard_template = "comm_subject_listboard_template"
    listboard_url = "comm_subject_listboard_url"
    navbar_selected_item = "comm_followup"
    search_form_url = "comm_subject_listboard_url"
    listboard_fa_icon = "fas fa-users-between-lines fa-2x"
    model_wrapper_cls = SubjectConsentModelWrapper
    name_search_field: str = "legal_name"
    identity_regex: str = r"^[A-Z0-9\ ]+$"

    def get_queryset(self):
        # TODO: expand filter??
        qs = super().get_queryset()
        return qs.filter(group_identifier__isnull=False)

    def extra_search_options(self, search_term) -> List[Q]:
        q_objects = super().extra_search_options(search_term)
        if re.match(r"^[A-Za-z\-]+$", search_term):
            q_objects.append(Q(familiar_name__exact=search_term))
        if re.match(r"^[0-9\-]+$", search_term):
            q_objects.append(Q(group_identifier__exact=search_term))
        return q_objects



from edc_subject_dashboard.views import SubjectDashboardView


class DashboardView(SubjectDashboardView):

    consent_model = "intecomm_consent.subjectconsent"
    navbar_selected_item = "consented_subject"
    visit_model = "intecomm_subject.subjectvisit"
    history_button_label = "Audit"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update(subject_listboard_url="screen_group_url")
        return context
